@@ -2,7 +2,7 @@
 ## src/common/optparser.py
 ##
 ## Copyright (C) 2003-2005 Vincent Hanquez <tab AT snarc.org>
-## Copyright (C) 2003-2012 Yann Leboulanger <asterix AT lagaule.org>
+## Copyright (C) 2003-2014 Yann Leboulanger <asterix AT lagaule.org>
 ## Copyright (C) 2005-2006 Dimitur Kirov <dkirov AT gmail.com>
 ##                         Nikos Kouremenos <kourem AT gmail.com>
 ## Copyright (C) 2006-2008 Jean-Marie Traissard <jim AT lapin.org>
@@ -38,6 +38,9 @@ from common import caps_cache
 import sqlite3 as sqlite
 import logger
 
+import logging
+log = logging.getLogger('gajim.c.optparser')
+
 class OptionsParser:
     def __init__(self, filename):
         self.__filename = filename
@@ -56,14 +59,18 @@ class OptionsParser:
         new_version = gajim.config.get('version')
         new_version = new_version.split('-', 1)[0]
         seen = set()
-        regex = re.compile(r"(?P<optname>[^.]+)(?:(?:\.(?P<key>.+))?\.(?P<subname>[^.]+))?\s=\s(?P<value>.*)")
+        regex = re.compile(r"(?P<optname>[^.=]+)(?:(?:\.(?P<key>.+))?\.(?P<subname>[^.=]+))?\s=\s(?P<value>.*)")
 
         for line in fd:
             try:
                 line = line.decode('utf-8')
             except UnicodeDecodeError:
                 line = line.decode(locale.getpreferredencoding())
-            optname, key, subname, value = regex.match(line).groups()
+            match = regex.match(line)
+            if match is None:
+                log.warn('Invalid configuration line, ignoring it: %s', line)
+                continue
+            optname, key, subname, value = match.groups()
             if key is None:
                 self.old_values[optname] = value
                 gajim.config.set(optname, value)
@@ -85,7 +92,6 @@ class OptionsParser:
     def write_line(self, fd, opt, parents, value):
         if value is None:
             return
-        value = value[1]
         # convert to utf8 before writing to file if needed
         if isinstance(value, unicode):
             value = value.encode('utf-8')
@@ -226,6 +232,8 @@ class OptionsParser:
             self.update_config_to_01401()
         if old < [0, 14, 90, 0] and new >= [0, 14, 90, 0]:
             self.update_config_to_014900()
+        if old < [0, 16, 0, 1] and new >= [0, 16, 0, 1]:
+            self.update_config_to_01601()
 
         gajim.logger.init_vars()
         gajim.logger.attach_cache_database()
@@ -913,3 +921,11 @@ class OptionsParser:
             gajim.config.set('use_stun_server', False)
         if os.name == 'nt':
             gajim.config.set('autodetect_browser_mailer', True)
+
+    def update_config_to_01601(self):
+        if 'last_mam_id' in self.old_values:
+            last_mam_id = self.old_values['last_mam_id']
+            for account in gajim.config.get_per('accounts'):
+                gajim.config.set_per('accounts', account, 'last_mam_id',
+                    last_mam_id)
+        gajim.config.set('version', '0.16.0.1')
